import unittest

from pyramid import testing
from pyramid.url import static_url

class Dummy(object):
    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)

    def get(self, value, default):
        return self.__dict__.get(value, default)


class TestMacrosSubscriber(unittest.TestCase):
    def setUp(self):
        self.config = testing.setUp()
        self.config.begin()

    def tearDown(self):
        self.config.end()

    def test_it(self):
        from jslibs import jslibs_macros_subscriber
        self.config.testing_add_renderer("jslibs:jslibs.pt")
        event = Dummy(request=testing.DummyRequest())
        jslibs_macros_subscriber(event)
        self.assertTrue(hasattr(event.request, 'jslibs'))


class TestIncludeMe(unittest.TestCase):
    def setUp(self):
        self.config = testing.setUp()
        self.config.begin()

    def tearDown(self):
        self.config.end()

    def test_it_default(self):
        from jslibs import includeme
        request = testing.DummyRequest()
        includeme(self.config)
        url = static_url(u'jslibs:resources/', request)
        self.assertEqual(url, u'http://example.com/jslibs_static/')

    def test_it_with_name(self):
        from jslibs import includeme
        request = testing.DummyRequest()
        self.config.registry.settings['jslibs.static_view_name'] = 'aname'
        includeme(self.config)
        url = static_url(u'jslibs:resources/', request)
        self.assertEqual(url, u'http://example.com/aname/')





from pyramid.events import NewRequest
from pyramid.events import subscriber
from pyramid.renderers import get_renderer

def includeme(config):
    """
    A hook for inclusion in a Pyramid app. You can set a
    static_view_name by assigning a value to
    ``jslibs.static_view_name`` in settings Defaults to:
    ``u"jslibs_static"``

    config
        An instance of ``pyramid.config.Configurator`` from the calling app.


    Example:

    .. code-block:: python

        ...

        def main(global_config, **settings):
            ...
            config = Configurator(...)
            config.scan('myapp')
            config.include('jslibs')
            ...
    """
    view_name = config.registry.settings.get(u'jslibs.static_view_name',
                                             u'jslibs_static')
    config.add_static_view(view_name, u'jslibs:resources/')
    config.scan('jslibs')

@subscriber(NewRequest)
def jslibs_macros_subscriber(event):
    """
    Adds the macros to the request object so it can then be called in
    a template like so:

    .. code-block:: xml

          <metal:resources metal:use-macro="request.jslibs['google-cdn']" />
    """
    r = get_renderer('jslibs:jslibs.pt')
    event.request.jslibs = r.implementation().macros




# Activate pyramid support, in case pyramid is present.
# Silently not do it, if pyramid is not installed.

try:
    import pyramid
    pyramid = pyramid   # shut up pylint
except ImportError:
    pass
else:
    import pyramid_support
    pyramid_support = pyramid_support # shut up pylint





import sys
import os
import subprocess
import jslibs

def _concat_path(fname, *rnames):
    return os.path.join(os.path.dirname(fname), *rnames)

def module_path(mod, *rnames):
    return _concat_path(mod.__file__, *rnames)


def update_submodule(dir):
    # update submodules in this tree.
    current_dir = os.getcwd()
    os.chdir(dir)
    try:
        subprocess.call(('git', 'submodule', 'init'))
        subprocess.call(('git', 'submodule', 'update'))
    finally:
        os.chdir(current_dir)


def prep_jquery(dir):
    # from jQuery Makefile:
    #@@sed '/EXPOSE/r src/sizzle-jquery.js' ${SIZZLE_DIR}/sizzle.js | grep -v window.Sizzle > ${SRC_DIR}/selector.js
    outf = file(os.path.join(dir, 'src', 'selector.js'), 'w')
    for line in file(os.path.join(dir, 'src', 'sizzle', 'sizzle.js')):
        if 'EXPOSE' in line:
            outf.write(file(os.path.join(dir, 'src', 'sizzle-jquery.js')).read())
        elif 'window.Sizzle' not in line:
            outf.write(line)
    outf.close()


def prep_tinymce(dir):
    # run 'ant' in the source directory
    current_dir = os.getcwd()
    os.chdir(dir)
    try:
        subprocess.call(('ant', ))
    finally:
        os.chdir(current_dir)


def prepare_sources():
    # Need to update the submodules.
    # It also prepares the sources for jQuery.
    # (Other sources do not need this.)
    jslibs_gitroot = module_path(jslibs, '..')
    externals_dir = module_path(jslibs, 'externals')

    update_submodule(jslibs_gitroot)

    # jQuery 1.6.2
    # it has its own submodules that requires an update
    update_submodule(os.path.join(externals_dir, 'jquery', '1.6.2'))
    # it also requires some special preparation
    prep_jquery(os.path.join(externals_dir, 'jquery', '1.6.2'))

    # TinyMCE 3.4.7
    # needs its main js precompiled
    prep_tinymce(os.path.join(externals_dir, 'tinymce', '3.4.7'))

    # TinyMCE 3.4.8
    # needs its main js precompiled
    prep_tinymce(os.path.join(externals_dir, 'tinymce', '3.4.8'))

    # TinyMCE 3.5b3
    # needs its main js precompiled
    prep_tinymce(os.path.join(externals_dir, 'tinymce', '3.5b3'))



def main(argv=sys.argv):
    if len(argv) > 1:
        raise RuntimeError, 'jslibs_prepare_sources accepts no parameters.'

    prepare_sources()

    print "\n\n##### All sources prepared OK"

if __name__ == '__main__':
    main()
